package com.jiheng.formservice.common.utils;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.*;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.List;

public class JacksonJsonUtil {
	private static final Logger logger = LoggerFactory.getLogger(JacksonJsonUtil.class);

	private static final ObjectMapper objectMapper = new ObjectMapper();
	
	static {

        SimpleModule simpleModule = new SimpleModule();
        simpleModule.addSerializer(Long.class, ToStringSerializer.instance);
        objectMapper.registerModule(simpleModule);
        objectMapper.configure(MapperFeature.PROPAGATE_TRANSIENT_MARKER, true);// 忽略 transient 修饰的属性

		objectMapper.setDateFormat(new SimpleDateFormat(DateUtils.DEFAULT_DATE_TIME_FORMAT));	// 设置日期解析格式
		objectMapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);	// 设置null值为不序列化
		objectMapper.configure(SerializationFeature.WRITE_NULL_MAP_VALUES, false);
	}
	
	public static ArrayNode getArrayNode() {
		return objectMapper.createArrayNode();
	}
	public static ObjectNode getObjectNode() {
		return objectMapper.createObjectNode();
	}
	/**
	 * 通过path选择JsonNode 可能返回一个MissingNode
	 * @param json
	 * {"name":"Jack","age":20,"class":["1","2","3"],"father":{"name":"old Jack","age":50}}
	 * father/name 可以选择到old Jack
	 * class/1 可以选举到"2"
	 * @return
	 * @throws Exception
	 */
	public static <T> T treeNodeToObject(JsonNode node,Class<T> classes) {
		T result = null;
		try {
			if(node!=null&&!node.isMissingNode()&&!node.isNull()) {
				result= objectMapper.treeToValue(node, classes);
			}
		} catch (Exception e) {
			logger.error("treeNodeToObject error:", e);
		}
		return result;
	}
	/**
	 * 通过path选择JsonNode 可能返回一个MissingNode
	 * @param json
	 * {"name":"Jack","age":20,"class":["1","2","3"],"father":{"name":"old Jack","age":50}}
	 * father/name 可以选择到old Jack
	 * class/2 可以选举到"2"
	 * @return
	 * @throws Exception
	 */
	public static JsonNode pathSelectNode(JsonNode node,String path) {
		JsonNode result = node;
		try {
                if(StringUtils.isBlank(path)) {
                    return node;
			}
			String[] paths = StringUtils.split(path, "/");
			for(String item:paths) {
				if(StringUtils.isNumeric(item)) {
					result = result.path(NumberUtils.toInt(item));
				}else {
					result = result.path(item);
				}
				if(result.isMissingNode()||result.isNull()) {
					break;
				}
			}
		} catch (Exception e) {
			logger.error("pathSelectNode error:", e);
		}
		return result;
	}
	/**
	 * 将JSON串解析成JsonNode并获取路径下的节点
	 * @param json
	 * 			JSON字符串
	 * @return
	 * @throws Exception
	 */
	public static JsonNode readTree(String json,String path) {
		JsonNode result = null;
		try {
			result = pathSelectNode(objectMapper.readTree(json),path);
		} catch (Exception e) {
			logger.error("parse json readTree error:", e);
		}
		return result;
	}
	/**
	 * 将JSON串解析成JsonNode
	 * @param json
	 * 			JSON字符串
	 * @return
	 * @throws Exception
	 */
	public static JsonNode readTree(String json) {
		return readTree(json,null);
	}
	/**
	 * 将JSON串解析成对象
	 * @param json
	 * 			JSON字符串
	 * @param classes
	 * 			对象类型
	 * @return
	 * @throws Exception
	 */
	public static <T> T readObject(String json, Class<T> classes) {
		T result = null;
		try {
			result = objectMapper.readValue(json, classes);
		} catch (Exception e) {
			logger.error("parse json object error:", e);
		}
		return result;
	}

	/**
	 * 转换类型
	 * @param json
	 * 			JSON字符串
	 * @param classes
	 * 			对象类型
	 * @return
	 * @throws Exception
	 */
	public static <T> T convertObject(Object source, Class<T> classes) {
		T result = null;
		try {
			result = objectMapper.readValue(objectMapper.writeValueAsString(source), classes);
		} catch (Exception e) {
			logger.error("parse json object error:",e);
		}
		return result;
	}
	/**
	 * 将JSON串解析成对象
	 * @param json
	 * 			JSON字符串
	 * @param classes
	 * 			对象类型
	 * @return
	 * @throws Exception
	 */
	public static <T> T readObject(File file, Class<T> classes) {
		T result = null;
		try {
			result = objectMapper.readValue(file, classes);
		} catch (Exception e) {
			logger.error("parse json object error:",e);
		}
		return result;
	}
	/**
	 * 将JSON串解析成对象
	 * @param json
	 * 			JSON字符串
	 * @param classes
	 * 			对象类型
	 * @return
	 * @throws Exception
	 */
	public static <T> T readObject(InputStream in, Class<T> classes) {
		T result = null;
		try {
			result = objectMapper.readValue(in, classes);
		} catch (Exception e) {
			logger.error("parse json object error:",e);
		}
		return result;
	}
	
	/**
	 * 将JSON串解析成对象
	 * @param json
	 * 			JSON字符串
	 * @param classes
	 * 			对象类型
	 * @return
	 * @throws Exception
	 */
	public static <T> List<T> readList(String jsonStr, Class<T> classes) {
		List<T> result = null;
		try {
			JavaType javaType = objectMapper.getTypeFactory().constructCollectionType(List.class, classes);
			result = objectMapper.readValue(jsonStr, javaType);
		} catch (Exception e) {
			logger.error("parse json object error:",e);
		}
		return result;
	}
	/**
	 * 将JSON串解析成对象
	 * @param json
	 * 			JSON字符串
	 * @param classes
	 * 			对象类型
	 * @return
	 * @throws Exception
	 */
	public static <T> List<T> readList(File file, Class<T> classes) {
		List<T> result = null;
		try {
			JavaType javaType = objectMapper.getTypeFactory().constructCollectionType(List.class, classes);
			result = objectMapper.readValue(file, javaType);
		} catch (Exception e) {
			logger.error("parse json object error:",e);
		}
		return result;
	}
	
	/**
	 * 将JSON串解析成对象
	 * @param json
	 * 			JSON字符串
	 * @param classes
	 * 			对象类型
	 * @return
	 * @throws Exception
	 */
	public static <T> List<T> readList(InputStream in, Class<T> classes) {
		List<T> result = null;
		try {
			JavaType javaType = objectMapper.getTypeFactory().constructCollectionType(List.class, classes);
			result = objectMapper.readValue(in, javaType);
		} catch (Exception e) {
			logger.error("parse json object error:",e);
		}
		return result;
	}
	
	/**
	 * 将对象串解析成JSON字符串
	 * @throws Exception
	 */
	public static String writeObject(Object object) {
		String result = null;
		try {
			result = objectMapper.writeValueAsString(object);
		} catch (Exception e) {
			logger.error("parse json object error:", e);
		}
		return result;
	}
	
	/**
	 * 将对象串写到文件
	 * @throws Exception
	 */
	public static void writeObject(File file,Object object) {
		try {
			objectMapper.writeValue(file,object);
		} catch (Exception e) {
			logger.error("parse json object error:", e);
		}
	}
	
	/**
	 * 将对象串写到文件
	 * @throws Exception
	 */
	public static void writeObject(OutputStream out,Object object) {
		try {
			objectMapper.writeValue(out,object);
		} catch (Exception e) {
			logger.error("parse json object error:", e);
		}
	}
	

}
