package com.jiheng.formservice.framework.aspect;

import cn.hutool.core.lang.UUID;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jiheng.formservice.common.context.BaseContextHandler;
import com.jiheng.formservice.common.utils.JacksonJsonUtil;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 通过切面 记录请求日志
 *
 * @author ch
 */
@Slf4j
@Aspect
@Component
public class SysLogAspect {

    private final ThreadLocal<LocalDateTime> currentThread = new ThreadLocal<>();
    @Autowired
    private ObjectMapper objectMapper;
    
    @Pointcut("@annotation(io.swagger.annotations.ApiOperation)")
    public void logPointCut() {
    }

    @Before("logPointCut() && @annotation(apiOperation)")
    public void invokeBefore(JoinPoint joinPoint, ApiOperation apiOperation) {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = requestAttributes.getRequest();
        log.info("\033[1;33m========================================" + Thread.currentThread().getName() + "请求开始========================================\033[0m");
        try {
            log.info("\033[1;33m请求地址:<" + request.getMethod() + "> " + request.getRequestURL().toString() + "\033[0m");
            log.info("\033[1;33m请求描述:" + apiOperation.value() + "\033[0m");
            log.info("\033[1;33m请求方法:" + joinPoint.getSignature() + "\033[0m");
            if ("GET".equals(request.getMethod())) {
                log.info("\033[1;33m请求参数:" + JSONObject.toJSONString(request.getQueryString() == null ? "无请求参数" : request.getQueryString().replace("&", "、")) + "\033[0m");
            } else {
                List<Object> args = Arrays.stream(joinPoint.getArgs()).filter(x -> !String.valueOf(x).contains("org.apache")).collect(Collectors.toList());
                log.info("\033[1;33m请求参数:" + objectMapper.writeValueAsString(args) + "\033[0m");
            }
            currentThread.set(LocalDateTime.now());
        } catch (Exception e) {
            log.info("\033[1;33m日志切面出现错误:{}\033[0m", e.getMessage());
        }
    }

    @AfterReturning(returning = "obj", pointcut = "logPointCut()")
    public void invokeAfter(Object obj) {
        if (currentThread.get() != null) {
            //log.info("\033[1;33m响应参数:" + (obj == null ? "无响应参数" : JSONObject.toJSONString(obj)) + "\033[0m");
            log.info("\033[1;33m本次耗时:" + Duration.between(currentThread.get(), LocalDateTime.now()).toMillis() + "毫秒\033[0m");
            log.info("\033[1;33m========================================" + Thread.currentThread().getName() + "响应结束========================================\033[0m");
        }
    }
    
    @Pointcut("@within(org.springframework.web.bind.annotation.RestController)")
	public void logPointAround() {
	}
    
    @Around("logPointAround()")
    public Object doAroundLog(ProceedingJoinPoint thisJoinPoint) throws Throwable{
        Object r = null;
        String method = thisJoinPoint.getSignature().getName();
        LocalDateTime startTime = LocalDateTime.now();
        String uuid = UUID.fastUUID().toString();
        String url = null;
        String userId = null;
        String userName = null;
        String tenant = null;
        Map param = null;
        boolean flag = false;
        try {
        	ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
            HttpServletRequest request = requestAttributes.getRequest();
            userId = request.getHeader("userId");
            userName = request.getHeader("userName");
            tenant = request.getHeader("tenant");
            url = request.getRequestURL().toString();

            BaseContextHandler.setUserId(userId);
            BaseContextHandler.setName(userName);
            BaseContextHandler.setTenant(tenant);

            param = getArgs(thisJoinPoint,((MethodSignature)thisJoinPoint.getSignature()).getParameterNames());
            log.info("\033[1;33m[访问地址:{}] [方法:{}] [{}] [userId:{}] [userName:{}][tenant:{}][参数:{}]\033[0m",url,method,uuid,userId,userName,tenant,param);
            r = thisJoinPoint.proceed ();//被代理对象执行结果
           
        } catch (Throwable e) {
        	flag = true;
            throw e;//异常信息有统一异常处理器处理
        }
        log.info("\033[1;33m[访问地址:{}] [方法:{}] [{}] [执行时间：{}(豪秒)] [{}]\033[0m",url,method,uuid,Duration.between(startTime, LocalDateTime.now()).toMillis(),flag?"异常":"正常");
        return r;
    }
    private static String types = "java.lang.Integer,java.lang.Double,java.lang.Float,java.lang.Long,java.lang.Short,java.lang.Byte,java.lang.Boolean,java.lang.Char,java.lang.String,int,double,long,short,byte,boolean,char,float";
    
    private Map getArgs(ProceedingJoinPoint joinPoint,String[] parameterNames) {
    	Map allArgs = new HashMap();
    	Object[] args = joinPoint.getArgs();
    	 for (int k = 0; k < args.length; k++) {
    	       Object arg = args[k];
    	       if(arg==null) {
    	    	   continue;
    	       }
    	       if(arg instanceof ServletResponse||arg instanceof ServletRequest) {//
    	    	continue;   
    	       }else if(arg instanceof MultipartFile) {//文件
    	    	   allArgs.put(parameterNames[k],((MultipartFile)arg).getOriginalFilename());
    	       }else if(arg instanceof MultipartFile[]) {//文件
    	    	   allArgs.put(parameterNames[k],((MultipartFile[])arg).length);
    	       }else if(arg instanceof Number||arg instanceof String||arg instanceof Character||arg instanceof Boolean) {//简单类型
    	    	   allArgs.put(parameterNames[k],arg);
    	       }else if(arg instanceof Collection) {//集合
    	    	   allArgs.put(parameterNames[k],arg);
    	       }else {//复杂类型
    	    	   Map m = JacksonJsonUtil.readObject(JacksonJsonUtil.writeObject(arg), Map.class);
    	    	   if(m!=null) {
    	    		   allArgs.putAll(m);
    	    	   }
    	       }
    	 }
    	return allArgs;
    }
}

